#!/usr/bin/env python3

'''EnvSetup logging utilities.'''

import sys
import re

RED = '\033[91m'
GREEN = '\033[92m'
YELLOW = '\033[93m'
BLUE = '\033[94m'
DEF = '\033[0m'


def log(text: str, error: bool = False):
    '''Output log message to stout or stderr.

    :param text: Message to log
    :type text: str
    :param error: Wether it should output to stderr or not, defaults to False
    :type error: bool, optional
    '''

    fo = sys.stderr if error else sys.stdout
    print(text, file=fo)
    fo.flush()


def info(message: str):
    '''Print formatted info message.

    :param message: Message to print
    :type message: str
    '''

    log(' {}ℹ{} {}'.format(BLUE, DEF, message))


def success(message: str):
    '''Print formatted success message.

    :param message: Message to print
    :type message: str
    '''

    log(' {}✔{} {}'.format(GREEN, DEF, message))


def warning(message: str):
    '''Print formatted warning message.

    :param message: Message to print
    :type message: str
    '''

    log(' {}⚠{} {}'.format(YELLOW, DEF, message), True)


def error(message: str):
    '''Print formatted error message.

    :param message: Message to print
    :type message: str
    '''

    log(' {}✖{} {}'.format(RED, DEF, message), True)


def strip_colors(text: str):
    '''Drop console colors.

    :param text: Text in which colors should be dropped
    :type text: str

    :return: Cleaned text
    :rtype: str
    '''
    return re.sub(r'\033\[[\d;]+m', '', text)


def escape_html(text: str):
    '''Escape HTML tags in a text.

    :param text: Text to clean
    :type text: str

    :return: Cleaned text
    :rtype: str
    '''
    html = text.strip()
    html = html.replace('<', '&lt;')
    html = html.replace('>', '&gt;')
    html = html.replace('\033[90m', '<span style="color: gray;">')
    html = html.replace('\033[91m', '<span style="color: red;">')
    html = html.replace('\033[92m', '<span style="color: green;">')
    html = html.replace('\033[93m', '<span style="color: orange;">')
    html = html.replace('\033[94m', '<span style="color: blue;">')
    html = html.replace('\033[95m', '<span style="color: purple;">')
    html = strip_colors(html)
    return html
